<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

require_once('classes/common_gui.php');
require_once('include/data.inc.php');
require_once('include/changelog.inc.php');

define('ENTRIES_PER_PAGE', 100);

if (!isset($console_access)) {
	$page->html_start();
}

function build_filter() {
	$filter = array();
	$sql = array();
	$filter_enabled = false;
	if( !empty( $_REQUEST['match_username'] ) ) {
		$filter['match_username'] = $_REQUEST['match_username'];

		$likestr = nacoma_db::quote_string('%'.$filter['match_username'].'%');
		$sql[] = 'username LIKE '.$likestr;

		$filter_enabled = true;
	}
	if( !empty( $_REQUEST['match_object'] ) ) {
		$filter['match_object'] = $_REQUEST['match_object'];

		$likestr = nacoma_db::quote_string('%'.$filter['match_object'].'%');
		$query  = '(object_name LIKE '.$likestr;
		$query .= ' OR oldname LIKE '.$likestr;
		$query .= ' OR newname LIKE '.$likestr;
		$query .= ')';
		$sql[] = $query;

		$filter_enabled = true;
	}
	if( !empty( $_REQUEST['match_time_start'] ) ) {
		$filter['match_time_start'] = $_REQUEST['match_time_start'];
		$sql[] = 'time >= '.nacoma_db::quote_string($filter['match_time_start']);

		$filter_enabled = true;
	}
	if( !empty( $_REQUEST['match_time_stop'] ) ) {
		$filter['match_time_stop'] = $_REQUEST['match_time_stop'];
		$sql[] = 'time <= '.nacoma_db::quote_string($filter['match_time_stop']);

		$filter_enabled = true;
	}

	return array(
			'querystring' => http_build_query($filter),
			'sql' => implode(' AND ', $sql),
			'fields' => $filter,
			'enabled' => $filter_enabled
	);
}

function print_pager($page_number, $filter=array()) {
	global $n_pages;
	if ($n_pages <= 1)
		return;
	echo '<div class="pagination">';

	echo '<form action="changelog_history.php" method="get">';
	echo 'Goto:';
	echo '<input type="text" name="page" value="'.$page_number.'" style="margin: 0 1em;"/>';
	foreach( $filter['fields'] as $field => $value ) {
		echo '<input type="hidden" name="'.$field.'" value="'.htmlentities($value).'" />';
	}

	if ($page_number > 1)
		echo '<a href="changelog_history.php?page='.($page_number-1).'&'.$filter['querystring'].'"><img src="images/icons/arrow-left.png" alt="Previous" title="Previous" /></a>';
	else
		echo '<img src="images/icons/arrow-left.png" alt="Previous" title="Previous" />';
	for($i=max(1,$page_number-5); $i<=$n_pages && $i<=$page_number+5;$i++) {
		if ($i == $page_number)
			echo "<strong>$page_number</strong>";
		else
			echo '<a href="changelog_history.php?page='.$i.'&'.$filter['querystring'].'">'.$i.'</a>';
	}
	if ($page_number < $n_pages)
		echo '<a href="changelog_history.php?page='.($page_number+1).'&'.$filter['querystring'].'"><img src="images/icons/arrow-right.png" alt="Next" title="Next" /></a>';
	else
		echo '<img src="images/icons/arrow-right.png" alt="Next" title="Previous" />';
	echo '</form>';
	echo '</div>';
}

function changelog_history($page, $filter=array()) {
	if (!$page)
		$page = 1;
	$changestr=null;
	$other_users = array();
	$rights = get_rights();
	$user = get_user();
	if (isset($rights['configuration_all'])) {
		$query = 'SELECT * FROM changelog_history WHERE 1=1';
	} else {
		$query = 'SELECT * FROM changelog_history ' .
				'WHERE (object_type, object_name) IN ' .
				'(SELECT object_type, object_name FROM changelog_history ' .
				'WHERE username = '.nacoma_db::quote_string($user).')';
	}
	if( !empty($filter['sql']) ) {
		$query .= ' AND ' . $filter['sql'];
	}


	$query .= ' ORDER BY time DESC';
	$query .= ' LIMIT '.ENTRIES_PER_PAGE.' OFFSET '.(($page-1)*ENTRIES_PER_PAGE);
	$result = nacoma_db::exec_query($query);
	$row = false;
	foreach ($result->fetchAll(PDO::FETCH_ASSOC) as $ary) {
		$row['action'] = '';
		$row['object_type'] = $ary['object_type'];
		if (!$ary['oldstate'] && !$ary['newstate'])
			continue;
		else if ((!$ary['oldname'] && $ary['newname']) || !$ary['oldstate']) {
			$row['object'] = $ary['newname'];
			$row['action'] .= "Created new object <b>{$ary['newname']}</b>";
			if ($ary['username']) {
				$row['username'] = "{$ary['username']}";
				$changestr.=" by <b>{$ary['username']}</b>";
				$other_users[] = $ary['username'];
			}
			$row['time'] = $ary['time'];
			$changes[] = $row;
			continue;
		} elseif (!$ary['newstate']) {
			$row['object'] = $ary['oldname'];
			$row['action'] .="Deleted object <b>{$ary['oldname']}</b>";
			if ($ary['username']) {
				$row['username'] = "{$ary['username']}";
				$other_users[] = $ary['username'];
			}
			$row['time'] = $ary['time'];
			$changes[] = $row;
			continue;
		}

		if (!isset($row['time']) && !empty($ary['time'])) {
			$row['time'] = $ary['time'];
		}

		$objchanges=null;
		$pre = convert_changelog_object($ary['object_type'], $ary['oldstate']);
		$post = convert_changelog_object($ary['object_type'], $ary['newstate']);
		$row['object'] = !empty($ary['newname']) ? $ary['newname'] : $ary['oldname'];
		if ($ary['oldname'] != $ary['newname']) {
			$row['action'] .= "Renamed object from '<b>{$ary['oldname']}</b>' to '<b>{$ary['newname']}</b>'.<br />";
		}

		if ($pre === false || $post === false) {
			$row['action'] .=" Unknown changes to <b>{$ary['newname']}</b> (can't tell you exactly what - the changelog history seems corrupt).<br />";
		}
		else {
			foreach ($pre as $oldkey => $oldval) {
				if (isset($post[$oldkey]) && $post[$oldkey] == $oldval) {
					unset($post[$oldkey]);
					continue;
				}

				if (is_array($oldval) || (isset($post[$oldkey]) && is_array($post[$oldkey]))) {
					if (is_array($oldval) && is_array($post[$oldkey])) {
						foreach ($oldval as $ok => $ov) {
							foreach ($post[$oldkey] as $nk => $nv) {
								if ($ov == $nv) {
									unset($oldval[$ok]);
									unset($post[$oldkey][$nk]);
								}
							}
						}
					}

					if (!empty($post[$oldkey])) {
						foreach($post[$oldkey] as $key => $value) {
							$post[$oldkey][$key] = htmlspecialchars($value);
						}
						$row['action'] .= "Added member(s) <b>".implode(', ', $post[$oldkey])."</b> to attribute <b>$oldkey</b>.<br />";
					}
					if (!empty($oldval)) {
						foreach($oldval as $key => $value) {
							$oldval[$key] = htmlspecialchars($value);
						}
						$row['action'] .= "Removed member(s) <b>".implode(', ', $oldval)."</b> from attribute <b>$oldkey</b>.<br />";
					}
					unset($post[$oldkey]);
				}
				else if (!isset($post[$oldkey])) {
					$row['action'] .= "Removed attribute <b>".htmlspecialchars($oldkey)."</b>.<br />";
				}
				else {
					$row['action'] .= "Changed attribute <b>".htmlspecialchars($oldkey)."</b> to '<b>".htmlspecialchars($post[$oldkey])."</b>'";
					if (isset($pre[$oldkey]) && !empty($pre[$oldkey])) {
						$row['action'] .= " from ".htmlspecialchars($pre[$oldkey]);
					}
					$row['action'] .= ".<br />";
					unset($post[$oldkey]);
				}
			}

			foreach ($post as $newkey => $newval) {
				$row['action'] .= "<li>Added attribute <b>".htmlspecialchars($newkey)."</b> with value <b>".htmlspecialchars($newval)."</b>.<br />";
			}
		}

		if ($ary['username']) {
			$other_users[] = $row['username'] = htmlspecialchars($ary['username']);
		}
		$changes[] = $row;
		unset($row);
	}

	if (!empty($changes)) {
		$changestr = '';
		$i = 0;
		foreach ($changes as $row) {
			# create the tr
			$object = $row['object'];
			if (strstr($object, ';')) {
				$obj_parts = explode(';', $object);
				if (sizeof($obj_parts) == 2) {
					$object = $obj_parts[1] . " on host ".$obj_parts[0];
				}
			}
			$changestr .= '<tr>';
			$changestr .= '<td valign="top">'.$row['username'].'</td>';
			$changestr .= '<td valign="top">'.$row['object_type'].'</td>';
			$changestr .= '<td valign="top">'.$object.'</td>';
			$changestr .= '<td>'.(isset($row['action']) ? $row['action'] : "&nbsp;").'</td>';
			$changestr .= '<td valign="top" style="border-right: 1px solid #dcdccd;">'.$row['time'].'</td>';
			$changestr .= '</tr>';
		}
	}

	if (!empty($changestr))
		return $changestr;
	else
		return false;
}

$filter = build_filter();

$sql = 'SELECT COUNT(*) FROM changelog_history';
if( !empty($filter['sql']) ) {
	$sql .= ' WHERE '.$filter['sql'];
}

$res = nacoma_db::exec_query($sql);
$n_pages = $res->fetch(PDO::FETCH_NUM);
$n_pages = ceil($n_pages[0] / ENTRIES_PER_PAGE);
unset($res);

$page_number = isset($_GET['page'])?$_GET['page']:1;
if ($page_number > $n_pages)
	$page_number = $n_pages;
if ($page_number < 1)
	$page_number = 1;

$changes = changelog_history($page_number, $filter);

echo '<h2>Historical configuration changes</h2>';
echo '</div>';
echo "<div class=\"object-table\">\n";
print_pager($page_number, $filter);

echo '<a id="toggle_filter_bar" href="#">Toggle filter bar</a>';

echo '<form action="?" method="get">';
echo '<table class="changelog max zebra ObjectTable"><thead>';
echo '<tr class="header">';
echo '<th class="CmdBar">User</th>';
echo '<th class="CmdBar">Object type</th>';
echo '<th class="CmdBar">Object</th>';
echo '<th class="CmdBar">Action</th>';
echo '<th class="CmdBar">Time</th>';
echo '</tr>';

if( $filter['enabled'] ) {
	echo '<tr class="header" id="filterbar">';
} else {
	echo '<tr class="header" id="filterbar" style="display: none;">';
}
echo '<th>';
echo 'Filter user name:<br /><input type="text" value="'.htmlentities(isset($_REQUEST['match_username'])?$_REQUEST['match_username']:'', ENT_QUOTES, 'UTF-8').'" name="match_username" /><br /><input type="submit" value="Filter" />';
echo '</th>';
echo '<th></th>';
echo '<th>';
echo 'Filter object name:<br /><input type="text" value="'.htmlentities(isset($_REQUEST['match_object'])?$_REQUEST['match_object']:'', ENT_QUOTES, 'UTF-8').'" name="match_object" /><br /><input type="submit" value="Filter" />';
echo '</th>';
echo '<th></th>';
echo '<th>';
echo 'Filter start time:<br /><input type="text" value="'.htmlentities(isset($_REQUEST['match_time_start'])?$_REQUEST['match_time_start']:'', ENT_QUOTES, 'UTF-8').'" name="match_time_start" /><br />';
echo 'Filter end time:<br /><input type="text" value="'.htmlentities(isset($_REQUEST['match_time_stop'])?$_REQUEST['match_time_stop']:'', ENT_QUOTES, 'UTF-8').'" name="match_time_stop" /><br />';
echo '<input type="submit" value="Filter" />';
echo '</th>';
echo '</tr>';

echo '</thead>';
if (!empty($changes)) {
	echo '<tbody>'.$changes.'</tbody>';
} else {
	echo '<tbody><tr><td colspan="5">No changes detected</td></tr></tbody>';
}
echo '</table></div>';
echo '</form>';

print_pager($page_number, $filter);
echo '</div>';

echo <<<ENDOFSCRIPT
<script type="text/javascript">
$(document).on('click', '#toggle_filter_bar', function(e) {
	e.preventDefault();
	$('#filterbar').toggle();
	return false;
});
</script>
ENDOFSCRIPT;

$page->html_end();

?>
