<?php

if(!defined('SKIP_KOHANA')) {
	// edit.php is mostly loaded directly, but it is required from
	// websiterunner.php from some tests, so we cannot rely on SKIP_KOHANA
	// not already being set
	define('SKIP_KOHANA', true);
	require('/opt/monitor/op5/ninja/index.php');
}

require_once('classes/common_gui.php');
require_once('include/changelog.inc.php');
require_once('op5/log.php');
//Debug purposes
$log = Op5Log::instance('auth');

function print_propagate_selection($obj, $page, $input)
{
	$any_mselect = false;
	$prop_vars = empty($_REQUEST['prop_vars']) ? array() : $_REQUEST['prop_vars'];
	if (!$prop_vars) {
		echo "No variables selected to propagate. Choose the options you want to apply to other objects by checking the boxes to the far right in the form.\n";
	}
	echo "<p>Select which " . $obj->type . " objects you want to propagate these settings to:</p></div><div class=\"object-table\">\n";

	$page->form_start();
	$page->hidden_var('obj_type', $obj->type);
	$denorm = $obj->denormalize();
	echo '<table class="max zebra ObjTable">'."\n";
	echo '<tr class="header"><td colspan="3">Propagate</td></tr>';
	$i = 0;
	foreach ($prop_vars as $k => $discard) {
		$i++;
		if (isset($denorm[$k])) {
			$value = $denorm[$k];
			if (is_array($value))
				$value = join(',', $value);
			else if (is_bool($denorm[$k]))
				$value = var_export($denorm[$k], true);
		}
		else {
			if (isset($obj->properties['var_index'][$k])) {
				$value = '';
				$denorm[$k] = '';
			}
			else {
				echo "You cannot modify variable keys to custom variables and propagate them at the same time.<br />\n";
				exit;
			}
		}
		echo "<tr><td class='HelpList'><a href=\"#\" title=\"Click to view help on ".$k."\" tabindex='-1' data-help='".$k."' style=\"border: 0px\"><img src=\"images/icons/shield-help.png\" alt=\"Help\" title=\"Help\" /></a></td><td class=\"VarList\">$k</td><td>$value</td></tr>\n";
		if ($denorm[$k] === false) {
			$page->hidden_var("prop_vars[$k]", '0');
		} elseif (!is_array($denorm[$k])) {
			$page->hidden_var("prop_vars[$k]", $denorm[$k]);
		} else {
			foreach ($denorm[$k] as $foo => $bar) {
				$page->hidden_var("prop_vars[$k][$foo]", $bar);
			}
		}
		if (!$any_mselect && isset($obj->properties['var_index'][$k]) && $obj->properties['var_index'][$k] === 'mselect')
			$any_mselect = true;
	}
	$i++;
	echo "<tr><td class='HelpList'><a href=\"#\" title=\"Click to view help on object selection\" tabindex='-1' data-help='object_selection' style=\"border: 0px\"><img src=\"images/icons/shield-help.png\" alt=\"Help\" title=\"Help\" /></a></td><td class=\"VarList\">" .
		str_replace('_', ' ', ucfirst($obj->type)) .
		' objects</td><td class="List'.(($i%2) == 0 ? 'Odd' : 'Even').'">';
	if ($obj->type === 'service' || $obj->type === 'serviceescalation')
		$page->make_multi_service_selection();
	elseif ($obj->type === 'host' || $obj->type === 'hostescalation')
		$page->make_multi_host_selection();
	else
		$page->make_multi_selection($obj->type, false);

	echo '</td></tr>';
	if ($any_mselect) {
		echo "<tr><td class='HelpList'><a href=\"#\" title=\"Click to view help on propagating multi-value attributes\" tabindex='-1' data-help='propagation_format_multi_value_attributes' style=\"border: 0px\"><img src=\"images/icons/shield-help.png\" alt=\"Help\" title=\"Help\" /></a></td><td class=\"VarList\">";
		echo '<label for="prop_type">Propagation format for multi-value attributes</label>';
		echo '</td><td>';
		echo '<select name="prop_type" id="prop_type"><option value="replace" selected="selected">Replace</option><option value="append">Append</option><option value="subtract">Subtract</option></select>';
		echo '</td></tr>';
	}
	echo "</table>\n";
	$page->submit_button(false, 'action_dummy');
	$page->hidden_var('action', 'prop2');
	$page->hidden_var('obj_id', $obj->id);
	$page->form_end();
	echo '</div>';
	$page->html_end();
}

if (!$input->obj_type) {
	require_once('index.php');
	exit(0);
}

nacoma_objlock::die_if_locked_hard();

$page->import_if_changes();

if ($input->obj_type==='host' && (!$input->obj_id || $input->obj_id==='new') &&
	$input->get_string('list')!='true' && empty($input->host) && $input->action!='prop2' && empty($input->obj)){
	header('location: host_wizard.php');
	exit;
}

# make sure the passed object type is actually supported
# (that is, there is a class and a class-file for it)
$class_name = $input->obj_type . '_nacoma_Model';
if (in_array($input->obj_type, $input->protected) || !(class_exists($class_name) && is_subclass_of($class_name, 'object_nacoma_Model'))) {
	$page->html_start("Oops");
	bug("I don't know how to handle objects of the type '$input->obj_type'.");
	$page->html_end();
	exit(0);
}

if ($input->action === 'New')
	$input->obj_id = '';

$obj = new $class_name($input->obj_id);

if ($input->master_obj_type && (!$input->obj_id || $input->obj_id == 'new'))
	$obj->set_master_object_type($input->master_obj_type);

$clone_id = $input->get_numeric('clone_id');
if ($clone_id !== false)
	$obj->clone_id = $clone_id;

if (empty($obj->obj)) {
	$obj->id = 'new';
	$obj->get_default_object();
}

# if we can't compute the master object id we're most likely
# dealing with a new object, so prime it with the master object
# id the we got from the user
if (!$obj->compute_master_obj_id() && $input->master_obj_id)
  $obj->set_master_object_id($input->master_obj_id);

# Now it should be safe to use the object type as page topic :)
$page->html_start(ucfirst(str_replace('_', ' ', $obj->type)));

if ($input->obj_type === 'serviceescalation' || $input->obj_type === 'hostescalation') {
	$rightsc = get_nacoma_access('contact');
	$rightscg = get_nacoma_access('contactgroup');
	if (!in_array(NACOMA_ACCESS_VIEW_ALL, $rightsc) || !in_array(NACOMA_ACCESS_VIEW_ALL, $rightscg)) {
		$page->draw_object_selector($obj);
		deny_access("You are not allowed to use this feature because you haven't got access to view contacts/contactgroups.");
	}
}

# If the object exists, see if we have permissions to it.
if (!$obj->can_user_read($obj->id)) {
	$page->draw_object_selector($obj);
	if (!$input->obj_id) {
		$page->html_end();
	} else if ($input->obj_id == 'new') {
		deny_access("You are not allowed to create {$input->obj_type} objects");
	} else {
		deny_access("You are not allowed to view this {$input->obj_type} object");
	}
}

# The object type has a parent object, so we must be allowed to add relations
# to it, whether this object exists yet or not.
if ($obj->master_obj_type && $obj->master_obj_id) {
	$master_nacoma_Model = $obj->master_obj_type . '_nacoma_Model';
	if (!$master_nacoma_Model::can_user_read($obj->master_obj_id)) {
		$page->draw_object_selector($obj);
		if (!$obj->id || $obj->id == 'new')
			deny_access("You are not allowed to add {$input->obj_type} objects to this {$obj->master_obj_type}.");
		else
			deny_access("You are not allowed to edit this {$input->obj_type} object, or it doesn't exist.");
	}
}

switch ($input->action) {
	# 'Submit' is used when someone has made changes to the
	# object on the screen and pressed "apply changes". It
	# *should* mean we have an object passed to us from the
	# input_nacoma_Model. If it doesn't, something's wrong
 case 'Submit':
 case 'Submit anyway':
    
    if (isset($input->obj['hostgroup_name']) && !$input->get_string('force') && $_REQUEST['poller']) {
		
			print <<<END
			<div></div>
			<img src="images/icons/shield-warning_24x24.png" alt="" class="shield_warning" />
			<h1>Requested Hostgroup Rename Action</h1>
			<form method="POST" action="">
				<p>You may not rename this hostgroup_name for the following reasons:</p>
		        <p>CRITICAL: This poller hostgroup "{$input->obj['hostgroup_name']}" requires additional configuration in other configuration location </p>	
				{$input->as_form_elements()}
				<input type="hidden" name="force" value="force" />
			
				<input type="submit" name="action" value="Submit anyway" />
				<input type="submit" name="action" value="back" />
			</form>
END;
			$page->html_end();
		}


	$obj->obj = array();
	$obj->normalize($input->obj);
	$obj->custom_vars = $input->custom_vars[$obj->id];
	if (!$input->get_string('force') && has_others_changed($input->obj_type, $input->obj_id)) {
		$changes = new ChangeLog_nacoma_Model($_SESSION['nacoma_last_change'][$obj->type.'/'.$obj->id], $input->obj_id);
		if (count($changes)) {
			$changestr = $changes->get_html_list();
			print <<<END
			<div></div>
			<img src="images/icons/shield-warning_24x24.png" alt="" class="shield_warning" />
			<h1>Ooops!</h1>
			<form method="POST" action="">
				<p>Someone else changed this {$input->obj_type} object while you were editing:</p>
				{$changestr}
				{$input->as_form_elements()}
				<input type="hidden" name="force" value="force" />
				<p>Your changes will <b>replace</b> those of the other user(s) - are you sure?</p>
				<input type="submit" name="action" value="Submit anyway" />
				<input type="submit" name="action" value="Undo your changes" />
			</form>
END;
			$page->html_end();
		}
	}
	if ($obj->can_user_write($input->obj_id)) {
		$log->log('debug', 'Write permitted for ' . get_user() . ': ' . $input->obj_type . ' with id ' . $input->obj_id);
		$erray = $obj->save_object();
		if ($erray !== true) {

			# This is only ever important when input->obj_id is 'new',
			# but we might as well always do it
			$obj->set_object_id($input->obj_id);

			echo "Failed to save the object, for the following reasons:";
			$page->print_validation_error_table($obj);

			# since we might have a completely bogus object here, we
			# must make sure all required variables are listed in it,
			# so add those that might be missing.
			# Note that save_object() returns AFTER template expansion
			# occurs, but BEFORE template collapsing (on errors, that is),
			# so we *know* required variables that have gone missing here
			# can be safely set to an empty value before we print it
			foreach ($obj->properties['required'] as $k) {
				if (!isset($obj->obj[$k])) {
					$obj->obj[$k] = '';
				}
			}

			# now it's safe to collapse the object back, removing
			# template-inherited variables
			$obj->template_collapse_object();
		}
		else {
			echo '<p><img src="images/icons/shield-ok_16x16.png" alt="" style="margin-bottom: -3px" /> ';
			echo "<strong>Successfully changed 1 $obj->type object</strong></p>\n";
		}

	}
	else {
		$log->log('debug', 'No write permission for ' . get_user() . ': ' . $input->obj_type . ' with id ' . $input->obj_id);
		if ($input->obj_id === 'new')
			deny_access("You are not allowed to create new {$input->obj_type} objects.");
		else
			deny_access("You are not allowed to edit this {$input->obj_type} object.");
	}
	break;
	# expand is what happens when someone presses the 'Advanced'
	# link in the command bar for an object.
 case 'expand':
	$obj->expand_object();
	break;

 # propagate means we should copy one or more selected variables
 # to one or more other objects of the same type. We'll never
 # copy the object identifier, and it's impossible to copy
 # hidden variables
 case 'Propagate selected settings':
	if (!empty($input->obj))
		$obj->normalize($input->obj);
	print_propagate_selection($obj, $page, $input);
	exit;
	break;

 case 'prop2':
	$obj_list = array_pop($input->selection);

	$prop_vars = empty($_REQUEST['prop_vars']) ? array() : $_REQUEST['prop_vars'];
	if (empty($prop_vars)) {
		echo '<p><img src="images/icons/shield-critical_16x16.png" alt="" style="margin-bottom: -3px" /> ';
		echo "<strong>No variables to propagate.</strong></p>\n";
	}
	if (empty($obj_list)) {
		echo '<p><img src="images/icons/shield-critical_16x16.png" alt="" style="margin-bottom: -3px" /> ';
		echo "<strong>No objects selected to propagate to.</strong></p>\n";
	}

	$prop_type = 'replace';
	if (isset($_REQUEST['prop_type']) && !empty($_REQUEST['prop_type']))
		$prop_type = $_REQUEST['prop_type'];

	$erray = false;
	if (!empty($prop_vars) && !empty($obj_list))
		$erray = $obj->propagate_variables($prop_vars, $obj_list, $prop_type);

	if ($erray === true) {
		echo '<p><img src="images/icons/shield-ok_16x16.png" alt="" style="margin-bottom: -3px" /> ';
		echo "<strong>Successfully propagated " . count($prop_vars) .
			" variables to " . count($obj_list) . " " . $obj->type . " objects.</strong></p>\n";
	} elseif (is_array($erray)) {
		echo '<p><img src="images/icons/shield-warning.png" alt="" style="margin-bottom: -3px" /> ';
		echo "<strong>There were validation errors.<br />\n";
		foreach ($erray as $foo => $ary) {
			echo "$foo<br />\n";
			$page->print_validation_error_table(false, $ary);
		}
		echo '</strong></p>';
	} else {
		echo '<p><img src="images/icons/shield-critical_16x16.png" alt="" style="margin-bottom: -3px" /> ';
		echo '<strong>Fatal error. Bailing out.</strong></p>';
	}

	break;

	# clone with a link back to this page only happens
	# when we do a truly trivial clone (ie, single
	# object, no master objects, no nothing), so
	# the various objects that *can* be cloned like this
	# should sort everything out in their clone_object
	# method.
 case 'clone':
	$obj = $obj->copy_object();
	break;
 # 'Go' is used when someone picks a new object from the
 # quickjump dropdown. No special action is required for
 # it to work properly
 case 'Go': default:
	break;
}

if ($obj->id != 'new') {
	store_last_viewed($obj->type, $obj->id);
}

if ($owner = nacoma_objlock::get_lock_owner($obj->type, $obj->id)) {
	// Only show warning if user is other than current. Otherwise a warning
	// will always be shown the second time the page is reloaded.
	if ($owner != op5Auth::instance()->get_user()->get_username()) {
		print <<<END
			<img src="images/icons/shield-warning_24x24.png" alt="" style="display: inline; float: left; margin-right: 10px;" />
			<h1>This object is currently being edited by $owner</h1>
			<p>Editing it right now is probably not a good idea. You should go away and come back later</p>
END;
	}
}
if ($obj->id != 'new') {
	print "<script type=\"text/javascript\">lock_obj_id=\"". addslashes($obj->id) ."\"; lock_obj_type='$obj->type'</script>";
}

if (isset($obj->parent_autodiscovered) && $obj->parent_autodiscovered) {
	print '<p><img src="images/icons/shield-warning.png" alt="" style="display: inline; float: left; margin-right: 10px;" />
		<strong>You were trying to reach a host service, but it is actually a hostgroup service</strong></p>';
}


// regex + selectbox for existing objects
$page->draw_object_selector($obj);

echo $page->help_for_object($obj);

if ($obj->type==='host' || $obj->type === 'hostgroup');
	echo '<br /><br />'.$page->print_services_link($obj);
echo "</div>";

# Draw the table where the user can actually edit the object
# we're showing
$page->form_start('foobar_form');
if (!empty($obj->master_obj_id)) {
	$page->hidden_var('master_obj_id', $obj->master_obj_id);
	$page->hidden_var('master_obj_type', $obj->master_obj_type);
}

echo top_bar_nacoma_Model::add_unsaved_warning();

// draws all of the form's content, e.g. the settings of the object to be manipulated
$page->draw_object($obj);
if (!$obj->can_user_write($obj->id)) {
	echo top_bar_nacoma_Model::add_warning('You are not authorized to edit this object.');
} else {
	$page->submit_button();
}

$page->submit_button('Propagate selected settings', 'action', false, array(
	'class' => 'propagate-settings',
	'style' => 'display:none;'));
$page->form_end();
$page->html_end();
