#!/usr/bin/php
<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

require_once('/opt/monitor/op5/nacoma/include/config.inc.php');
give_infinite_rights();

require_once('/opt/monitor/op5/nacoma/include/import.inc.php');
require_once('/opt/monitor/op5/nacoma/include/export.inc.php');
require_once('/opt/monitor/op5/nacoma/api/api.inc.php');

$ocache = false;
$debug = False;
$oconf_prefix = '/opt/monitor/etc/oconf';

function prefixcmp($str, $prefix)
{
	return strcmp(substr($str, 0, strlen($prefix)), $prefix);
}

function get_param($str)
{
	$ary = preg_split('/=/', $str, 2);
	return $ary[1];
}

function set_files($prefix = 'etc/oconf')
{
	global $Supported_Object_Types;
	global $HOME;
	if ($prefix[0] != '/')
		$prefix = $HOME.'/'.$prefix;
	@mkdir($prefix, 0777, true);

	echo "Creating virtual configuration files in nacoma db.\n";
	nacoma_db::exec_query('TRUNCATE TABLE file_tbl');
	$id = 0;
	foreach ($Supported_Object_Types as $otype) {
		if ($otype == 'new_host' || $otype == 'new_service' || $otype == 'FILE')
			continue;
		$id++;
		touch("$prefix/$otype.cfg");
		nacoma_db::exec_query("INSERT INTO file_tbl(id, file_name) " .
					   "VALUES($id, '$prefix/$otype.cfg')");
		nacoma_db::exec_query("UPDATE $otype SET file_id = $id");
	}
}

function usage($msg=False)
{
	if ($msg !== False)
		echo "$msg\n";
	echo "Usage: import-reduce [options]\n\n";
	echo "Where options can be any combination of the following:\n";
	echo "  --cfg-file=</path/to/object-config-file>\n";
	echo "         This can be used to specify template config files, fe\n";
	echo "  --object-cache=</path/to/objects.precache>\n";
	echo "  --actions=[s]etfiles,[t]emplatereduce,[h]ostgroupify-services,[i]mport-only\n";
	echo "\nThis program will take an objects.precache (or objects.cache) file\n";
	echo "and a file containing templates, import the objects.precache file into the\n";
	echo "Nacoma database and then apply each template one by one to see\n";
	echo "which one reduces the objects the most. It's intended as a helper\n";
	echo "to import 3rd party configuration files into Nacoma.\n";
	echo "\nWarning: This should not be run as any user other than monitor\n";
	if ($msg !== False)
		exit(1);
	exit(0);
}

function load_templates($otype)
{
	$t_nacoma_Model_name = $otype . '_template_nacoma_Model';

	$res = nacoma_db::exec_query("SELECT id, name FROM $otype WHERE name IS NOT NULL");
	$templates = array();
	while ($row = $res->fetch()) {
		$templates[$row[0]] = array();
		$tobj = new $t_nacoma_Model_name($row[0]);
		$obj = $tobj->obj;
		# unset un-interesting variables
		unset($obj['id']);
		unset($obj['name']);
		unset($obj['check_command']);
		unset($obj['register']);
		unset($obj['service_description']);
		$templates[$row[0]]['class'] = $tobj;
		$templates[$row[0]]['kvvec'] = $obj;
	}
	if (!count($templates)) {
		return False;
	}

	foreach ($templates as $tid => $ary) {
		$tobj = $ary['class'];
		$obj = $ary['kvvec'];
		printf("Template %s has %d kv pairs\n", $tobj->obj['name'], count($obj));
		$query = "SELECT count(1) FROM $otype WHERE register = 1";
		foreach ($obj as $k => $v) {
			$query .= " AND $k = '$v'";
		}
		#echo "$query;\n";
	}

	return $templates;
}

# Reduces configuration by recursively applying a bunch of
# templates to a particular object type and checking which
# template causes the object to shrink as much as possible.
function reduce_otype_config($otype)
{
	global $debug;

	$class_name = $otype . '_nacoma_Model';
	$res = nacoma_db::exec_query('SELECT COUNT(1) FROM ' . $otype . ' WHERE register = 1');
	$row = $res->fetch();
	unset($res);
	$ocount = intval($row[0]);
	if (!$ocount) {
		echo "No $otype objects to reduce. Weird stuff.\n";
		return;
	}
	$templates = load_templates($otype);
	if ($templates === False) {
		echo "  !!! No templates available for $otype objects.\n";
		return;
	}
	return;

	printf("  Reducing $ocount $otype object%s with %d templates\n",
		   $ocount > 1 ? "s" : "", count($templates));

	$res = nacoma_db::exec_query("SELECT id FROM $otype WHERE register = 1");
	$rows = $res->fetchAll();
	unset($res);
	foreach ($rows as $row) {
		$obj = new $class_name($row[0]);
		$best_template = -1;
		#$obj->template_expand_object();
		$orig_count = $low = count($obj->obj);
		#printf("$otype '%s' has %d variables\n", $obj->name(), $low);
		foreach ($templates as $t_id => $t_name) {
			$obj->obj['template'] = $t_id;
			#echo "Trying template $t_name\n";
			$obj->template_collapse_object();
			#printf("  Template %s gives %d vars\n",
			#	   $templates[$t_id], count($obj->obj));
			if ($best_template === -1 || count($obj->obj) < $low) {
				$best_template = $t_id;
				$low = count($obj->obj);
				$best_name = $templates[$t_id];
			#	echo "  ### $t_name is best so far\n";
			}
			$obj->template_expand_object();
		}
		#printf("  ===  Best template is %s with %d vars\n",
		#	   $templates[$best_template], $low);
		$obj->obj['template'] = $best_template;
		$obj->save_object();
		if ($debug) {
			printf(" %s '%s' reduced by %d. Used '%s'\n",
			       ucfirst($otype), $obj->name(), $orig_count - $low, $best_name);
		}
	}
}

function reduce_config()
{
	$otypes = array('contact', 'host', 'service');
	echo "Reducing configuration using templates\n";
	foreach ($otypes as $otype)
		reduce_otype_config($otype);
}

function concat_keylist($keylist)
{
	return join(',";",', $keylist);
}

function mk_hostgroupify_query($keylist, $filter=False)
{
	$query = "SELECT * FROM (" .
		"SELECT count(1) cnt, id, service_description, check_command_args, " .
			"concat(" . concat_keylist($keylist) . ") conc " .
		"FROM service WHERE name IS NULL AND hostgroup_name IS NULL " .
		"GROUP BY conc ORDER BY cnt) subq " .
	"WHERE cnt > 5 AND conc IS NOT NULL";

	if ($filter)
		$query .= " AND $filter";

	return $query;
}

function hostgroupify_services()
{
	$keylist = array('service_description', 'template', 'check_command');
	$keylists = array($keylist);
	$queries = array(mk_hostgroupify_query($keylist, "check_command_args IS NULL"));
	$conc_ary = array(concat_keylist($keylist));
	$keylist[] = 'check_command_args';
	$keylists[] = $keylist;
	$conc_ary[] = concat_keylist($keylist);
	$queries[] = mk_hostgroupify_query($keylist);

	$total_savings = 0;
	$srvlist = array();
	foreach ($queries as $x => $query) {
		echo "x = $x\n";
		$srvlist[$x] = array();
		$srvlist[$x] = array();
		$savings = 0;
		echo "$query\n";
		$res = nacoma_db::exec_query($query);
		while ($o = nacoma_db::fetch_row($res, PDO::FETCH_OBJ)) {
			$srvlist[$x][$o->service_description] = $o;
			$savings += $o->cnt - 1;
		}
		echo "Savings: $savings\n";
		$total_savings += $savings;
	}

	echo("Total savings: $total_savings\n");
	#print_r($srvlist);
	foreach ($srvlist as $x => $ary) {
		foreach ($ary as $o) {
			$members = array();
			# fetch members first
			$query = 'SELECT DISTINCT host_name FROM service WHERE ' .
				'concat(' . concat_keylist($keylists[$x]) . ") = " .
				nacoma_db::quote_string($o->conc);
			$res = nacoma_db::exec_query($query);
			while ($row = nacoma_db::fetch_row($res)) {
				$members[] = $row[0];
			}
			$hg_name = '00s-hg-' . $o->service_description;
			$hg = new hostgroup_nacoma_Model($hg_name);
			$hg->obj['hostgroup_name'] = $hg_name;
			$hg->obj['members'] = $members;
			$hg->obj['alias'] = "Service-hostgroup for service $o->service_description";
			$hg->obj['file_id'] = 1;
			$error = $hg->save_object();
			$srv = new service_nacoma_Model($o->id);
			$srv->set_master_object_type('hostgroup');
			$srv->set_master_object_id($hg->id);
			$query = 'DELETE FROM service WHERE id != ' . $o->id . " AND " .
				'concat(' . concat_keylist($keylists[$x]) . ") = " .
				nacoma_db::quote_string($o->conc);
			$res = nacoma_db::exec_query($query);
			$srv->save_object();
		}
	}

	# clean up junction data
	$junctions = array('service_servicegroup',
	                   'service_contact', 'service_contactgroup');
	foreach ($junctions as $j) {
		nacoma_db::exec_query('DELETE FROM ' . $j . ' WHERE ' .
		               'service NOT IN(SELECT id FROM service)');
	}
}

define('ACTION_REDUCE', 1);
define('ACTION_SETFILES', 2);
define('ACTION_HOSTGROUPIFY', 3);
define('ACTION_IMPORT', 4); /* this is implied by all others */

$cfg_files = array();
# the default actions
$actions = ACTION_REDUCE | ACTION_IMPORT | ACTION_SETFILES;
for ($i = 1; $i < $argc; $i++) {
	$arg = $argv[$i];
	if (!prefixcmp($arg, "--cfg-file=")) {
		$cfg_files[] = get_param($arg);
		continue;
	}
	if (!prefixcmp($arg, "--object-cache=")) {
		$ocache = get_param($arg);
		continue;
	}
	if (!prefixcmp($arg, "--help")) {
		usage();
	}
	usage("Unknown argument: $argv[$i]\n");

	if (!prefixcmp($arg, "--actions=")) {
		$action_str = get_param($arg);
		$actions = 0;
		for($i = 0; $i < strlen($action_str); $i++) {
			if($action_str{$i} === 's')
				$actions |= ACTION_SETFILES;
			elseif ($action_str{$i} === 't')
				$actions |= ACTION_REDUCE;
			elseif ($action_str{$i} === 'h')
				$actions |= ACTION_HOSTGROUPIFY;
			elseif ($action_str{$i} === 'i')
				$action |= ACTION_IMPORT;
		}
		continue;
	}
}

if(empty($cfg_files) || !$ocache)
	usage();
$cfg_files[] = $ocache;

$all_files_exist = true;
foreach ($cfg_files as $file) {
	if (!file_exists($file)) {
		$all_files_exist = false;
		break;
	}
}

if (!$all_files_exist) {
	usage("Couldn't find file: $file\n");
}

if(!$actions) {
	usage("No valid actions specified\n");
}

printf("Importing configuration from:\n   %s\n", join("\n   ", $cfg_files));
import_config(posix_getuid(), $cfg_files);
if ($actions & ACTION_SETFILES)
	set_files($oconf_prefix);
if ($actions & ACTION_REDUCE)
	reduce_config();
if ($actions & ACTION_HOSTGROUPIFY)
	hostgroupify_services();
$erray = do_export(true);
$success = api_display_error($erray);
if ($success) {
	exit(0);
}
exit(1);
