<?php
/**
 * Input-related methods and variables.
 * This class automagically mangles a lot of input variables into a
 * manageable state.
 */
class input_nacoma_Model
{
	/** action */
	var $action = false;
	/** object id */
	var $obj_id = false;
	/** object */
	var $obj = false;
	/** object type */
	var $obj_type = false;
	/** host id */
	var $host_id = false;
	/** master object id */
	var $master_obj_id = false;
	/** master object type */
	var $master_obj_type = false;
	/** protected variables */
	var $protected = array('object', 'gui', 'help', 'escalation', 'pnp');
	/** user */
	var $user = false;
	/** custom variables */
	var $custom_vars = false;
	/**
	 * This variable will be populated with whatever selection
	 * targets we find in the input.
	 */
	var $selection = false;
	private $host_selection = false;
	private $service_selection = false;
	private $hostgroup_selection = false;
	private $selection_target = false;

	/**
	 * Parse input variables common to lots of pages
	 * @param $obj_type The object type we're dealing with
	 */
	public function __construct($obj_type = false)
	{
		global $DEBUG;

		$custom_vars = false;
		if (!empty($_REQUEST['action']))
		  $this->action = $_REQUEST['action'];

		if (!empty($_REQUEST['obj_type'])) {
			$this->obj_type = $_REQUEST['obj_type'];
			if (preg_match('/[^a-z_]/', $this->obj_type)) {
				$this->obj_type = false;
			}
		}

		if (isset($_REQUEST['master_obj_type']))
			$this->master_obj_type = trim($_REQUEST['master_obj_type']);
		if (isset($_REQUEST['host_id'])) {
			$this->master_obj_id = intval($_REQUEST['host_id']);
		}
		if (isset($_REQUEST['master_obj'])) {
			if (is_array($_REQUEST['master_obj'])) {
				foreach ($_REQUEST['master_obj'] as $type => $name) {
					if ($name) {
						$class = $type . '_nacoma_Model';
						$obj = new $class();
						$obj->get_object_by_name($name);
						$this->master_obj_id = intval($obj->id);
						$this->master_obj_type = $type;
						break;
					}
				}
			}
			else {
				$class = $this->master_obj_type . '_nacoma_Model';
				$obj = new $class();
				$obj->get_object_by_name($name);
				$this->master_obj_id = intval($obj->id);
			}
		}
		if (isset($_REQUEST['master_obj_id'])) {
			if (is_array($_REQUEST['master_obj_id'])) {
				foreach ($_REQUEST['master_obj_id'] as $type => $id) {
					if (intval($id)) {
						$this->master_obj_id = intval($id);
						$this->master_obj_type = trim($type);
						break;
					}
				}
			}
			else {
				$this->master_obj_id = intval($_REQUEST['master_obj_id']);
			}
		}

		if (isset($_REQUEST['obj_id']) && $_REQUEST['obj_id'])
		  $this->obj_id = $_REQUEST['obj_id'];
		elseif (isset($_REQUEST['obj_num']) && $_REQUEST['obj_num'])
		  $this->obj_id = $_REQUEST['obj_num'];
		# make it possible to pass object names from GUI
		elseif (isset($_REQUEST['service']) && is_string($_REQUEST['service']))
			$this->obj_id = $_REQUEST['host'] . ';' . $_REQUEST['service'];
		elseif (isset($_REQUEST['host']) && is_string($_REQUEST['host']))
			$this->obj_id = $_REQUEST['host'];
		elseif (isset($_REQUEST['obj_name'])) {
			$class = preg_replace('/^new_/', null, $this->obj_type) . '_nacoma_Model';
			$obj = new $class();
			if ($this->master_obj_type) {
				$obj->set_master_object_type($this->master_obj_type);
			}
			if ($this->master_obj_id) {
				$obj->set_master_object_id($this->master_obj_id);
			}
			$obj->get_object_by_name($_REQUEST['obj_name']);
			if ($obj->id !== false)
				$this->obj_id = $obj->id;
		}

		# parse custom variables
		if ($this->obj_type && !empty($_REQUEST['custom_vars'][$this->obj_type])) {
			$raw_custom_vars = $_REQUEST['custom_vars'][$this->obj_type];
			foreach ($raw_custom_vars as $id => $ary_list) {
				foreach ($ary_list as $ary) {
					assert('isset($ary["key"]) && isset($ary["value"])');
					# the stupid way of deleting a custom var
					# is (currently) simply deleting either the
					# key or the value
					if (empty($ary['key']) || strval($ary['value']) === "")
						continue;
					$custom_vars[$id][$ary['key']] = $ary['value'];
				}
			}
			$this->custom_vars = $custom_vars;
		}

		if ($this->obj_type && !empty($_REQUEST[$this->obj_type])) {
			$obj_list = $_REQUEST[$this->obj_type];

			if (!is_array($obj_list)) {
				$obj_list = false;
				return;
			}

			if (!$this->obj_id && count($obj_list) === 1) {
				$obj_keys = array_keys($obj_list);
				$this->obj_id = array_pop($obj_keys);
				$this->obj = $obj_list[$this->obj_id];
			}
		}

		$this->user = get_user();

		if (isset($_REQUEST['selection']))
			$this->selection['selection'] = $_REQUEST['selection'];

		if (isset($_REQUEST['selection_target'])) {
			$this->selection_target = $_REQUEST['selection_target'];
		}
		$this->selection['host'] = $this->parse_obj_selection('host');
		$this->selection['service'] = $this->parse_obj_selection('service');
		$this->selection['hostgroup'] = $this->parse_obj_selection('hostgroup');
		$this->post_parse_selection();
		foreach ($this->selection as $type => $ary)
			if (empty($ary))
				unset($this->selection[$type]);

		$exclude = $this->get_array('select_exclude');
		if (!empty($exclude)) {
			foreach ($exclude as $type => $exclusions) {
				foreach ($exclusions as $ex) {
					if (isset($this->selection[$type][$ex])) {
						unset($this->selection[$type][$ex]);
					}
				}
			}
		}
	}

	/**
	 * Given an object type, parses the selection based on dual
	 * inputs (selected_{$obj_type}s and selected_{$obj_type}groups)
	 * and concatenates them into a unique list of $obj_type
	 * id's in an $id => $id style indexed array.
	 */
	function parse_obj_selection($obj_type = 'host')
	{
		$group = $obj_type . 'group';
		$selection = array();

		$raw_selection = $this->get_array('selected_' . $obj_type . 's');
		if (is_array($raw_selection)) {
			$class = $obj_type . '_nacoma_Model';
			foreach ($raw_selection as $name) {
				$id = $class::get_id_for($name);
				if ($id === false)
					continue;
				$selection[$id] = $id;
			}
		}
		$groups = $this->get_array('selected_' . $obj_type. '_groups');
		if (empty($groups))
			return $selection;

		$junction = $obj_type . '_' . $obj_type . 'group';
		$groups = array_map('nacoma_db::quote_string', $groups);
		$query = "
			SELECT DISTINCT CASE
				WHEN (impl.id IS NULL) THEN $obj_type.id
				ELSE impl.id
			END AS id
			FROM $obj_type
			INNER JOIN $junction ON $obj_type.id = $junction.$obj_type
			INNER JOIN $group ON $junction.$group = $group.id
			LEFT JOIN $obj_type impl ON impl.template = $obj_type.id
			WHERE $group.{$group}_name = " . implode(" OR $group.{$group}_name = ", $groups);

		$result = nacoma_db::exec_query($query);
		foreach ($result->fetchAll(PDO::FETCH_NUM) as $row) {
			$class = $obj_type . '_nacoma_Model';
			if ($class::can_user_read($row[0]))
				$selection[$row[0]] = $row[0];
		}
		unset($result);
		return $selection;
	}

	/**
	 * Some special magic to deal with a host_selection being
	 * parsed when the actual target type are services.
	 */
	function post_parse_selection()
	{
		if (!$this->selection['service'] && !$this->selection['host'] && !$this->selection['hostgroup'])
			return;

		if ($this->selection_target !== 'service')
			return;

		# both host and service selection, so find all services
		# under the selected hosts, add them to the service
		# selection and remove the host selection
		if (!empty($this->selection['host'])) {
			$query = 'SELECT id FROM service WHERE host_name IN (' .
				join(',', $this->selection['host']) . ')';
			$result = nacoma_db::exec_query($query);
			while (($row = $result->fetch(PDO::FETCH_NUM))) {
				$this->selection['service'][$row[0]] = $row[0];
			}
			unset($result);
		}

		if (!empty($this->selection['hostgroup'])) {
			$query = 'SELECT id FROM service WHERE hostgroup_name IN('.
				join(',', $this->selection['hostgroup']) . ')';
			$result = nacoma_db::exec_query($query);
			while (($row = $result->fetch(PDO::FETCH_NUM))) {
				$this->selection['service'][$row[0]] = $row[0];
			}
			unset($result);
		}
	}

	/**
	 * Fetches an array from the $_REQUEST array
	 * @param $name The name of the array to fetch
	 * @return The array if it is an array, or false otherwise
	 */
	function get_array($name)
	{
		if (isset($_REQUEST[$name]) && is_array($_REQUEST[$name]))
		  return $_REQUEST[$name];

		return false;
	}

	/**
	 * Fetches a numeric variable from the $_REQUEST array
	 * @param $name The name of the numeric variable to fetch
	 * @return The variable if it is numeric, or false otherwise
	 */
	function get_numeric($name)
	{
		if (isset($_REQUEST[$name]) && is_numeric($_REQUEST[$name]))
		  return $_REQUEST[$name];

		return false;
	}

	/**
	 * Fetches a string variable from the $_REQUEST array
	 * @param $name The name of the string variable to fetch
	 * @return The variable if it is a string, or false otherwise
	 */
	function get_string($name)
	{
		if (isset($_REQUEST[$name]) && is_string($_REQUEST[$name]))
		  return $_REQUEST[$name];

		return false;
	}

	static private function array_to_form($array, $basename=false)
	{
		$str = '';
		foreach ($array as $key => $val) {
			if ($basename)
				$key = $basename.'['.$key.']';

			if (is_array($val))
				$str .= self::array_to_form($val, $key);
			elseif ($val)
				$str.= '<input type="hidden" name="'.$key.'" value="'.$val.'" />';
		}
		return $str;
	}

	/**
	 * Return a string of hidden form elements with all information retrieved
	 *
	 * Excluding 'action' to let the caller add it's own action buttons.
	 */
	function as_form_elements()
	{
		$elements = $_REQUEST;
		unset($elements['action']);
		return self::array_to_form($elements, false);
	}

	/**
	 * Print debugging information about the input variables
	 */
	function debug_print()
	{
		global $DEBUG;

		if (!$DEBUG)
		  return;

		echo "<pre>\n\$_REQUEST = ";
		print_r($_REQUEST);

		echo "mangled input variables = ";
		print_r(get_object_vars($this));

		echo "</pre>\n";
	}

	/**
	 * @param $services array
	 * @return string
	 */
	static function services_to_cmd_args(array $services) {
		return "'".implode(
			"' '",
			array_map(
				function($service) {
					return str_replace('$', '$$', $service);
				},
				$services
			)
		)."'";
	}
}
