<?php
require_once('op5/auth/Authorization.php');
require_once('op5/config.php');


/**
 * A group is the part of Monitor's ACL that glues together rights with users.
 * A user is part of one or more groups (she's always part of an implicit group
 * called "user_*username*"), and the groups in turn contain names of access rights
 * which all members inherits.
 */
class usergroup_nacoma_Model extends object_nacoma_Model
{
	public $lazy_write = false; /**< If true, don't automatically generate cgi.cfg - performance option when handling many users */
	public $properties = array(
		'var_index' => array(
			'name' => 'string',
			'access_rights' => 'mselect'
		),
		'default' => array(
			'access_rights' => array()
		),
		'required' => array(
			'name'
		),
		'special' => array(),
		'topic' => 'name'
	);

	/** type name */
	const TYPE = 'group';

	private static $groups = array();

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.usergroups";
	}

	function __construct($name = null)
	{
		$this->type = static::TYPE;
		$this->properties['special']['access_rights'] = array_reduce(Op5Authorization::get_all_auth_levels(), function($a, $b) {return array_merge($a, $b);}, array());;

		$this->get_groups();

		if ($name && isset(self::$groups[$name])) {
			$this->get_object_by_name($name);
		} else {
			$this->obj['name'] = $name;
		}

	}

	/**
	 * @return boolean
	 */
	function banish_object($user=true)
	{
		if(!isset($this->obj['name'])) {
			$this->delete_message = "You must set a name of the object you want to delete.";
			return false;
		}
		self::get_groups();
		if(!isset(self::$groups[$this->obj['name']])) {
			$this->delete_message = "There is no group called ".$this->obj['name'];
			return false;
		}
		unset(self::$groups[$this->obj['name']]);
		if (!$this->lazy_write) {
			try {
				self::persist();
				return true;
			} catch (RuntimeException $ex) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @return boolean
	 */
	function delete_object()
	{
		return $this->banish_object();
	}


	function get_default_object()
	{
		$this->obj = array();
		return $this->obj;
	}

	/**
	 * @return array of groupnames
	 */
	static function get_groups()
	{
		if( self::$groups ) {
			return self::$groups;
		}
		$groups = Op5Config::instance()->getConfig( 'auth_groups' );
		self::$groups = $groups;
		ksort(self::$groups);
		return self::$groups;
	}

	public static function get_objects_for_user($write = false)
	{
		return array_keys(self::get_groups());
	}

	/**
	 * @param $group
	 * @return array|false if object does not exist
	 */
	function get_object_by_name($group)
	{
		$groups = self::get_groups();
		$this->obj = array();
		if(!isset($groups[$group])) {
			return false;
		}
		foreach($groups[$group] as $key => $value) {
			$this->obj['access_rights'][$key] = $value;
		}
		$this->obj['name'] = $this->id = $this->obj['id'] = $group;
		return $this->obj;
	}

	function get_object_name()
	{
		return (isset($this->obj['name']) && $this->obj['name']) ? $this->obj['name'] : 'New Group';
	}

	/**
	 * @param $user boolean = true
	 * @return boolean
	 */
	function save_object($user = true)
	{
		if(!$this->validate_object() && $user) {
			return false;
		}
		self::get_groups();
		self::$groups[$this->obj['name']] = $this->obj['access_rights'];
		$this->id = $this->obj['id'] = $this->obj['name'];
		if(!$this->lazy_write) {
			try {
				self::persist();
				return true;
			} catch (RuntimeException $ex) {
				return false;
			}
		}
		return true;
	}

	function validate_object() {
		$result = true;
		if (!isset($this->obj['name']))
			$result = $this->validation_error('name', 'Group has no name');
		if (isset($this->obj['access_rights'])) {
			foreach($this->obj['access_rights'] as $access_right) {
				if(!isset($this->properties['special']['access_rights'][$access_right])) {
					$result = $this->validation_error($access_right, "Invalid access right");
				}
			}
		}
		return empty($this->validation_error_list);
	}

	/**
	 * @param $obj ignored
	 * @param $id string groupname
	 * @return boolean
	 */
	static function object_exists($id)
	{
		$groups = self::get_groups();
		return isset($groups[$id]);
	}

	/**
	 * Write to file
	 * @throws RuntimeException
	 */
	static function persist() {
		Op5Config::instance()->setConfig('auth_groups', self::get_groups());
	}

	function get_properties()
	{
		return $this->properties;
	}
}
