<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Users controller used for local user configuration
 * Requires authentication
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class LocalUsersException extends Exception {}
class Users_Controller extends Configuration_Controller {

	public function __construct () {

		parent::__construct();

		$this->template->toolbar->title = _("Local Users");

		$this->template->js[] = 'application/media/js/jquery.filterable.js';

		$this->template->css[] = 'application/media/css/jquery.filterable.css';
		$this->template->css[] = 'modules/nacoma/media/css/form.css';

		$this->template->content = new View('users');

		$this->linkprovider = LinkProvider::factory();
		$this->messages = new NoticeManager_Model();

		$this->template->content->messages = $this->messages;
		$this->template->content->linkprovider = $this->linkprovider;


	}

	/**
	 * Catch-all page for user configuration, performs different actions depending
	 * on what form button was pressed
	 */
	public function index () {

		$this->_verify_access('monitor.system.users:read');
		$this->_verify_access('monitor.system.usergroups:read');
		$this->template->content->users = UserPool_Model::all();

	}

	/**
	 * Create a new user from POST data
	 */
	public function create () {

		if (empty($_POST)) url::redirect($this->linkprovider->get_url('users'));
		$this->_verify_access('monitor.system.users:create');

		try {
			$user = new User_Model();
			$password = $this->input->post('password', '');
			$repeat = $this->input->post('password-repeat');

			if ($password !== $repeat) {
				throw new LocalUsersException(
					'Repeated password does not match password'
				);
			}

			$user->set_username($this->input->post('username'));
			$user->set_realname($this->input->post('realname', ''));
			$user->set_password($password);
			$user->set_groups($this->input->post('groups', array()));
			$user->set_modules($this->input->post('modules', array()));
			$user->save();

			$this->messages[] = new SuccessNotice_Model(
				'Created user ' . html::specialchars($user->get_username())
			);

		} catch (LocalUsersException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ORMDriverException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ORMException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ConfigException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		}

		$this->index();

	}

	/**
	 * Update an existing user from POST data
	 */
	public function update () {

		if (empty($_POST)) url::redirect($this->linkprovider->get_url('users'));
		$this->_verify_access('monitor.system.users:update');

		try {

			$username = $this->input->post('username');
			$user = UserPool_Model::all()->reduce_by('username', $username, '=')->one();

			$password = $this->input->post('password', '');
			$repeat = $this->input->post('password-repeat');

			if ($password !== $repeat) {
				throw new LocalUsersException(
					'Repeated password does not match password'
				);
			}

			$user->set_password($password);
			$user->set_realname($this->input->post('realname',''));
			$user->set_groups($this->input->post('groups', array()));
			$user->set_modules($this->input->post('modules', array()));
			$user->save();

			$this->messages[] = new SuccessNotice_Model(
				'Updated user ' . html::specialchars($user->get_username())
			);

		} catch (LocalUsersException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ORMDriverException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ORMException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ConfigException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		}


		$this->index();

	}

	/**
	 * Delete an existing user
	 */
	public function delete () {

		if (empty($_POST)) url::redirect($this->linkprovider->get_url('users'));
		$this->_verify_access('monitor.system.users:delete');

		try {

			$username = $this->input->post('username');
			$user = UserPool_Model::all()->reduce_by('username', $username, '=')->one();
			$user->delete();

			$this->messages[] = new SuccessNotice_Model(
				'Deleted user ' . html::specialchars($username)
			);

		} catch (LocalUsersException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ORMDriverException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ORMException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		} catch (ConfigException $e) {
			$this->messages[] = new ErrorNotice_Model($e->getMessage());
		}


		$this->index();

	}
}
